

// Copyright (C) 2000  LuckY
// Copyright (C) 2002  Antti Kirjavainen


/*LuckY* 12.16.00

  WildCard Compare Function --
  Allows you to compare strings using
  wildcards '*' and '?'

  Syntax Example:
    char *s1 = "abc*ghi";
    char *s2 = "abcdefghi";
    if(wildcmp(s1,s2))
      cout << "Success!";

  -- how it works --

  -> The first check: StartCmp <-
  If there is no * at the start of the searchstring, ie:
    str*
  the searchstring from its first letter to the first *
  or the last character must be identical to the letters
  of the sourcestring. So:
    str*
    sourcestring
  would be false because the 't' and 'o' do not match.

  -> The second check: EndCmp <-
  If there is no * at the end of the searchstring, ie:
    *str
  then all of the characters from the end of the search
  must be identical to the end of the sourcestring until
  the first character in the string is reached or a * is
  reached. So:
    *string
    sourcestring  
  would be true because all characters from end to the 
  first character or * match and the search string ends.

  -> The other checks <-
  Now that we have established that the head and tail (if
  without a *) match the sourcestring, we have to find the
  rest of the searchstring characters in the sourcestring.
  After the heads match up, we cut the head of the searchstring
  off, as we only have to deal with the rest. IE:
    s*string     <-- searchstring
    sourcestring <-- sourcestring
  the 's' matches both so the new strings are:
    *string
    sourcestring
  The same goes for the tail. So, after a:
    *urce*ring
    sourcestring
  we would then be left with
    *urce*
    sourcestring
  Then we'd do the final checks. We would simply check to see
  that all of the letters in the remaining string are also in
  the sourcestring in the same order. 
  The method for this is to build a searchstring character by
  character and make sure the new searchstring is in the
  sourcestring each time. IE:
    u <-new search string building using heapspace
    sourcestring
      ^          searchstring is found in sourcestring -> continue

    ur
    sourcestring
      ^^        found again...continue

    urc
    sourcestring
      ^^^       found again...continue
    urce
    sourcestring
      ^^^^      found again...out of characters..return true

  Ok, just one more thing. If there are multiple parts (meaning there
  are one or more * in the middle) of the remaining searchstring, ie:
    start searchstring  -> s*est*in*  
    after checking head -> *est*in*  (2 parts: 'est' and 'in')
  then we first check for the 'est' string by building it char by char
  (as shown above) then we reset the new searchstring to search the 
  next part char by char. IE:
    e <-new string in heapspace to use as search
    sourcestring
         ^
    es
    sourcestring
         ^^
    est
    sourcestring
         ^^^
    -- now we reset the search string to check for the second part --

    i <-since the first part was found, we need to find the second
    sourcestring
             ^
    in
    sourcestring
             ^^
  SUCCESS!

  lucky760@yahoo.com
*/
#include <iostream>
#include <cstring>
using namespace std;

bool wildcmp(const char*,const char*);
bool find(const char*,const char*);
bool wc_cmp(const char*,const char*,int);
bool StartCmp(char*,const char*);
bool EndCmp(char*,const char*);

/* the main wildcard compare function */
bool wildcmp(const char *findstr,const char *cmp) {
  bool retval = true;
  int len=strlen(findstr);

  //we alter the search string, so lets make a copy here
  //and alter it instead
  char *srch = new char[len+1];
  strcpy(srch,findstr);
  
  if (*srch != '*' && !StartCmp(srch,cmp)) 
    retval = false;
  else if (srch[len-1] != '*' && !EndCmp(srch,cmp))
    retval = false;                                
  else {
    //srch may have been changed in StartCmp and/or EndCmp
    //so we need to reset 'len' to the new length
    len = strlen(srch);

    //get space for the new search string
    char *s = new char[len+1];
    int j=0;

    for (int i=0;i<len && retval;++i) 
      if (srch[i] != '*') {
        //build the new search string
        s[j++] = srch[i];
        s[j] = '\0';

        //make sure it's found in the source string
        if (!find(s,cmp)) 
          retval = false;  
  
        //if we reach a * in the middle of the search 
        //string, reset string s to start building
        //the next part of the search string after the *
        if (srch[i+1] == '*') { 
          *s = '\0';
          j=0;
        }
      }
    delete [] s;
  }
  delete [] srch;
  return retval;
}

/* Helper Functions */

//check string for substring
bool find(const char *s1,const char *s2) {
  int srchlen=strlen(s1);
  int cmplen=strlen(s2);

  for (int i=0;i<cmplen;++i)
    if(wc_cmp(s1,s2+i,srchlen))
      return true;

  return false;
}

//replacement for strncmp, allows for '?' 
bool wc_cmp(const char *s1,const char *s2,int len) {
  for (int i=0;i<len;++i) 				
    if (s1[i] != s2[i] && s1[i] != '?')                  
      return false;                             					 
  return true;
}

//if there's no * at end of the search string, it
//is checked to be identical to the end of the source string
bool EndCmp(char *src,const char *cmp) {
  int slen=strlen(src);
  int clen=strlen(cmp);

  int j=slen-1;
  for (int i=clen-1;i>0 && src[j] != '*';--i,--j) 
    if(src[j] != cmp[i] && src[j] != '?')  
      return false;                        
  src[j] = '\0';

  return true;
}

//if there's no * at head of search string, it 
//is checked to be identical to the start of the source string
bool StartCmp(char *src,const char *cmp) {
  int len = strlen(cmp);

  int i=0;
  for ( ;i<len && src[i] != '*';++i) 
    if (src[i] != cmp[i] && src[i] != '?')
      return false;

  if (i < len) 
    strncpy(src,src+i,strlen(src)-i+1); 

  return true;
}