/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/


// Copyright (C) 2002  Farid Bengrid

#ifndef _FILEINFO_HPP_
#define _FILEINFO_HPP_

typedef struct
	{
	int day;
	int month;
	int year;
	int hour;
	int minute;
	int second;
	}structFileInfo_Date, *ptrstructFileInfo_Date;

#ifndef _FOLDERLIST_HPP_
#include "FolderList.hpp"
#endif



class FileInfo
	{
	public:
		/*
		* Contructor
		*/
		FileInfo(void);
		
		/*
		* Destructor
		*/
		~FileInfo(void);

		/*
		* Retrieve the file/Folder name has a long format string.
		* Return: string : The file/folder name.
		*/
		string getWideFilename(void);

		/*
		* Retrieve the file/Folder name has a short format string (dos 8.3).
		* Return: string : The file/folder name.
		*/
		string getShortFilename(void);

		/*
		* Check if the file is a directory.
		* Return: bool : true or false
		*/
		bool isDirectory(void);

		/*
		* Check if the file/directory has the archive flag set.
		* Applications use this attribute to mark files for backup or removal.
		* Return: bool : true or false
		*/
		bool isAnArchive(void);

		/*
		* Check if the file/directory is compressed.
		* For a directory, this means that compression is the default for newly created files and subdirectories
		* Return: bool : true or false
		*/
		bool isCompressed(void);

		/*
		* Check if the file/directory is encrypted.
		* For a file, this means that all data in the file is encrypted.
		* For a directory, this means that encryption is the default for newly created files and subdirectories.
		* Return: bool : true or false
		*/
		bool isEncrypted(void);

		/*
		* Check if the file/directory is hidden.
		* Return: bool : true or false
		*/
		bool isHidden(void);

		/*
		* Check if the file/directory is offline (not immediately available).
		* This attribute indicates that the file data has been physically moved to offline storage.
		* This attribute is used by Remote Storage, the hierarchical storage management software.
		* Applications should not arbitrarily change this attribute.
		* Return: bool : true or false
		*/
		bool isOffline(void);
		
		/*
		* Check if the file/directory can only be read and not modified.
		* Return: bool : true or false
		*/
		bool isReadOnly(void);

		/*
		* Check if the file/directory is system file.
		* For a directory, this means that compression is the default for newly created files and subdirectories
		* Return: bool : true or false
		*/
		bool isSystem(void);

		/*
		* Check if the file/directory is temporary file.
		* Return: bool : true or false
		*/
		bool isTemporary(void);

		/*
		* Return the local system date/time of the creation of the file/folder.
		* Return: structFileInfo_Date : structure containing the information (see FileInfo.hpp)
		*/
		structFileInfo_Date getDateCreate(void);
		
		/*
		* Return the local system date/time of the last access of the file/folder.
		* Return: structFileInfo_Date : structure containing the information (see FileInfo.hpp)
		*/
		structFileInfo_Date getDateLastAccessed(void);

		/*
		* Return the local system date/time of the last modification of the file/folder.
		* Return: structFileInfo_Date : structure containing the information (see FileInfo.hpp)
		*/
		structFileInfo_Date getDateLastWrite(void);

		/*
		* Convert the date information into a string.
		* Input: date    : the structure structFileInfo_Date containing the date information (see FileInfo.hpp).
		* Return: string : format is "DD-MM-YYYY"
		*/
		string dateToString(structFileInfo_Date date);
		
		/*
		* Convert the time information into a string.
		* Input: date    : the structure structFileInfo_Date containing the time information (see FileInfo.hpp).
		* Return: string : format is "HH:MM:SS"
		*/
		string timeToString(structFileInfo_Date date);
		
		/*
		* Retrieve the file size.
		* Return: ULARGE_INTEGER : The file size.
		*/
		ULARGE_INTEGER getFileSize(void);

	private:
		friend class FolderList;
		string wideFilename;
		string shortFilename;

		bool flag_isDirectory;
		bool flag_isAnArchive;
		bool flag_isCompressed;
		bool flag_isEncrypted;
		bool flag_isHidden;
		bool flag_isOffline;
		bool flag_isReadOnly;
		bool flag_isSystem;
		bool flag_isTemporary;
		
		structFileInfo_Date dateCreated;
		structFileInfo_Date dateLastAccessed;
		structFileInfo_Date dateLastWrite;
		
		ULARGE_INTEGER fileSize;
	};
#undef _FILEINFO_HPP_
#endif