/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/


// Copyright (C) 2002  Farid Bengrid


#include <stdlib.h>
#include <memory.h>
#include "GenericList.hpp"

/*
* Contructor
*/
GenericList::GenericList(void)
	{
	this->itemRoot   =NULL;
	this->itemLast   =NULL;
	this->nbItems    =0;
	this->cachedIndex=0;
	this->cachedItem =NULL;
	this->LastError  =0;
	}

/*
* Destructor
*/
GenericList::~GenericList(void)
	{
	clear();
	}

/*
* Add an element at the end of the list
* Input:  ptrData : A pointer to the data to store.
* Input:  dataSize: The number of bytes to be copied.
* Return: 0                : Successfull.
*         ERR_MEMORYFAILURE: Allocation memory failure
*         ERR_FUNCTIONPARAM: sizeData == 0 and ptrData!=NULL
*/
int GenericList::add(void * ptrData,size_t sizeData)
	{
	GenericListItem * Item=NULL;
	
	//Create a new Item and copy the data in it.
	if((Item=createNewItem(ptrData,sizeData))==NULL)	return this->LastError;

	//check the status of the list;
	if (this->itemRoot==NULL)
		{
		//The list is empty
		this->itemRoot=Item;
		this->itemLast=Item;
		}
	else
		{
		//Place the Item at the end of the list
		this->itemLast->setNextItem(Item);
		Item->setPreviousItem(this->itemLast);
		this->itemLast=Item;
		}
	this->nbItems++;

	
	//Set the cached Item to this one
	setCache(this->nbItems-1,Item);
	return 0;
	}

/*
* Insert a element at the position specified of the list
* Input : index: index of the insertion => 0 to this->Size()
* Input : ptrData : A pointer to the data to add
* Input : sizeData: number of bytes to be copied
* Return: 0: Successfull.
*         ERR_FUNCTIONPARAM: index <0 or (sizeData == 0 and ptrData!=NULL)
*         ERR_MEMORYFAILURE: Allocation memory failure
*/
int GenericList::insertAt(long index, void * ptrData, size_t sizeData)
	{
	GenericListItem * Item=NULL;
	GenericListItem * ItemToMove=NULL;

	//check Entry Parameters
	if (index<0)					return ERR_FUNCTIONPARAM;

	//Place the new element at the end of the list if required
	if (index>=this->nbItems)	return add(ptrData,sizeData);
	
	//Create a new Item and copy the data in it.
	if ((Item=createNewItem(ptrData,sizeData))==NULL)	return this->LastError;

	//Recover the item which will be moved after this new item
	ItemToMove=searchItem(index);

	//Reset the Previous Item so its next item is the new one.
	if (ItemToMove->getPreviousItem()!=NULL)
		{
		ItemToMove->getPreviousItem()->setNextItem(Item);
		Item->setPreviousItem(ItemToMove->getPreviousItem());
		}
	else
		{
		//There is no previous Item so the new Item will become root item of the list
		this->itemRoot=Item;
		}

	//Now the next field.
	Item->setNextItem(ItemToMove);
	ItemToMove->setPreviousItem(Item);

	//Increase the local count of items
	this->nbItems++;
	
	//Set the cached Item to this one
	setCache(index,Item);
	return 0;
	}

/*
* Update an Item with the content specified
* Input:  index  : the index of the item
* Input:  ptrData : A pointer to the data
* Input:  dataSize: The number of bytes of the data
* Return: 0                : Successfull.
*         ERR_FUNCTIONPARAM: Incorrect index specified or (datasize == 0 and ptrData!=NULL)
*         ERR_MEMORYFAILURE: Memory allocation failure
*/
int GenericList::update(long index, void * ptrData, size_t sizeData)
	{
	GenericListItem * Item=NULL;

	//Check validity of the parameters
	if (index<0 || index>=this->nbItems)	return ERR_FUNCTIONPARAM;

	//Retrieve the Item
	Item=searchItem(index);

	//Update the Content
	return Item->setItemData(ptrData,sizeData);
	}

/*
* Get a copy of the Item Data at the specified position
* Input:  index  : the index of the item
* Output: ptrData: A pointer to a pre-allocated memory where the data will be copied
* Return: 0                        : Successfull.
*         ERR_FUNCTIONPARAM        : Incorrect index specified, or ptrData==NULL
*         ERR_LISTITEM_MISSING_DATA: There are no stored data.
*/
int GenericList::get(long index, void * ptrData)
	{
	GenericListItem * Item=NULL;
	
	//Check validity of the parameters
	if (index<0 || index>=this->nbItems)	return ERR_FUNCTIONPARAM;
	if (ptrData==NULL)							return ERR_FUNCTIONPARAM;

	//Retrieve the Item
	Item=searchItem(index);

	//Get the Data
	if ((this->LastError=Item->getItemData(ptrData))==0)
		{
		//Set The Cache
		setCache(index,Item);
		}
	return this->LastError;
	}

/*
* Get a Pointer to the data for Read Access (DO NOT CHANGE THE CONTENT !!!)
* Input: index  : the index of the item
* Return: void *: Pointer to the Data
*           NULL: no Data,or index not found.
*/
void * GenericList::getPtrToData(long index)
	{
	GenericListItem * Item=NULL;
	void * ptrData=NULL;
	
	//Check validity of the parameters
	if (index<0 || index>=this->nbItems)	return NULL;

	//Retrieve the Item
	Item=searchItem(index);

	//Get the Data
	ptrData=Item->getPtrToData();

	//Set The Cache
	setCache(index,Item);
	
	return ptrData;
	}

/*
* Remove the Item at the index specified
* Input:  index: the index of the item
* Return: 0                : Successfull.
*         ERR_FUNCTIONPARAM: Incorrect index specified
*/
int GenericList::remove(long index)
	{
	GenericListItem * Item=NULL;

	if (index<0 || index>=this->nbItems)	return ERR_FUNCTIONPARAM;
	
	//Retrieve to item to remove
	Item=searchItem(index);

	//Remove it from the list
	extractFromList(Item);

	//Remove it from the memory;
	delete Item;
	
	//check if the item was the cached one. or a one before it
	if (isCached(index) || ((this->cachedIndex!=-1) && index<this->cachedIndex))
		{
		//yes so purge the cache.
		clearCache();
		}
	return 0;
	}

/*
* Remove the first element matching the one specified found in the list
* Input:  ptrData : A pointer to the data
* Input:  dataSize: The number of bytes of the data
* Return: 0                      : Successfull.
*         ERR_LIST_ITEM_NOT_FOUND: Item not found
*/
int GenericList::remove(void * ptrData,size_t sizeData)
	{
	GenericListItem * Item=NULL;
	long index;

	//Retrieve to item to remove
	if ((Item=searchItem(&index,ptrData,sizeData))==NULL)
		return ERR_LIST_ITEM_NOT_FOUND;

	//Remove it from the list
	extractFromList(Item);

	//Remove it from the memory;
	delete Item;
	
	//check if the item was the cached one. or a one before it
	if (isCached(index) || ((this->cachedIndex!=-1) && index<this->cachedIndex))
		{
		//yes so purge the cache.
		clearCache();
		}
	return 0;
	}

/*
Clear all the elements from the List
Input : none
Return: none
*/
void GenericList::clear(void)
	{
	GenericListItem * Item=NULL;
	while(this->itemRoot!=NULL)
		{
		Item=(this->itemRoot)->getNextItem();
		delete this->itemRoot;
		this->itemRoot=Item;
		}
	this->itemLast   =NULL;
	this->nbItems    =0;
	clearCache();
	}

/*
* Get the size in bytes of the element at the position specified
* Input: index  : Position of the element => 0 to this->Size()-1
* return: size_t: the number of bytes.
*              0: Item not found
*/
size_t GenericList::getDataSize(long index)
	{
	GenericListItem * Item=NULL;
	
	if (index<0 || index>=this->nbItems) return 0;

	Item=searchItem(index);

	setCache(index,Item);

	return Item->getItemSize();
	}

/*
* Return the index in the list of the first element matching the one specified
* Input:  ptrData : A pointer to the data
* Input:  dataSize: The number of bytes of the data
* Return: long                     : The index
*         ERR_LIST_ITEM_NOT_FOUND  : Item not found
*/
long GenericList::indexOf(void * ptrData,size_t sizeData)
	{
	GenericListItem * Item=NULL;
	long index;

	//Retrieve the item
	if ((Item=searchItem(&index,ptrData,sizeData))==NULL)
		return ERR_LIST_ITEM_NOT_FOUND;
	
	setCache(index,Item);
	return index;
	}

/*
* Check if there is in the list an element matching to the one specified,
* Input:  ptrData : A pointer to the data
* Input:  dataSize: The number of bytes of the data
* Return: true or false
*/
bool GenericList::contains(void * ptrData,size_t sizeData)
	{
	GenericListItem * Item=NULL;
	long index;

	//Retrieve the item
	if ((Item=searchItem(&index,ptrData,sizeData))==NULL)
		return false;
	
	setCache(index,Item);	
	return true;
	}

/*
* Return the number of elements in the list
* Input : none
* return: long : the number of elements
*/
long GenericList::size(void)
	{
	return this->nbItems;
	}

/*
* Create a new Item
* Input:  ptrData : A pointer to the data to store.
* Input:  dataSize: The number of bytes to be copied.
* Return: GenericListItem * : A pointer to the Item
*                       NULL: this->LastError=ERR_MEMORYFAILURE: Allocation memory failure
*                       NULL: This->LastError=ERR_FUNCTIONPARAM: sizeData == 0 and ptrData!=NULL
*/
GenericListItem * GenericList::createNewItem(void * ptrData,size_t sizeData)
	{
	GenericListItem * Item=NULL;

	//Create the Item;
	Item=new GenericListItem();
	if (Item==NULL)
		{
		this->LastError=ERR_MEMORYFAILURE;
		return NULL;
		}
	
	//Copy the data in it;
	if ((this->LastError=Item->setItemData(ptrData,sizeData))!=0)
		{
		delete Item;
		return NULL;
		}
	//Return it.
	return Item;
	}

/*
* Search an Item by its index
* Input:  index: The index
* Return: GenericListItem * : the Item
*/
GenericListItem * GenericList::searchItem(long index)
	{
	GenericListItem * Item=NULL;

	//Check if the item is the one in the cache
	if (isCached(index))
		{
		Item=this->cachedItem;
		}
	//If not do a iterative search
	else if (index<(this->nbItems/2))
		{
		//Start from the Root
		for(Item=this->itemRoot;index>0;index--,Item=Item->getNextItem());
		}
	else
		{
		//Start from the Last
		for(Item=this->itemLast;index<(this->nbItems-1);index++,Item=Item->getPreviousItem());
		}
	return Item;
	}

/*
* Search an Item by its content
* Input:  ptrData : A pointer to the data
* Input:  dataSize: The number of bytes of the data
* Output:    index: The index of the Item found
* Return: GenericListItem * : the Item
*                       NULL: Item not found
*/
GenericListItem * GenericList::searchItem(long * index,void * ptrData,size_t sizeData)
	{
	GenericListItem * Item=NULL;
	long idx;
	
	//check the cached value
	if (isCached(ptrData,sizeData))
		{
		*index=this->cachedIndex;
		return this->cachedItem;
		}

	for(Item=this->itemRoot,idx=0;idx<this->nbItems;Item=Item->getNextItem(),idx++)
		{
		if (Item->contains(ptrData,sizeData))
			{
			*index=idx;
			return Item;
			}
		}
	return NULL;
	}

/*
* Remove all the reference of an item from the list
* Input:  Item: The Item
* Return: none
*/
void GenericList::extractFromList(GenericListItem * item)
	{
	GenericListItem * itemPrev;
	GenericListItem * itemNext;

	//Recover the Previous and Next item
	itemPrev=item->getPreviousItem();
	itemNext=item->getNextItem();

	//is the Item to remove the root of the list ?
	if (itemPrev==NULL)
		{
		//yes
		//is the Item to remove also the last of the list ?
		if (itemNext==NULL)
			{
			//yes
			this->itemRoot=NULL;
			this->itemLast=NULL;
			}
		else
			{
			//no, it's only the root of the list
			this->itemRoot=itemNext;
			itemNext->setPreviousItem(NULL);
			}
		}
	//so is the Item to remove the last of the list ?
	else if (itemNext==NULL)
		{
		//yes
		this->itemLast=itemPrev;
		itemPrev->setNextItem(NULL);
		}
	else
		{
		//Well the item to remove is in the middle somewhere...
		itemPrev->setNextItem(itemNext);
		itemNext->setPreviousItem(itemPrev);
		}
	
	//Decrease the count of items
	this->nbItems--;
	}

/*
* Chech if the cached item correspond to the specified index
* Input:  index: the index looked for
* Return: true or false
*/
bool GenericList::isCached(long index)
	{
	if (this->cachedItem==NULL)	return false;
	if (this->cachedIndex!=index)	return false;
	return true;
	}

/*
* Chech if the cached item contain the specified data
* Input:  ptrData : A pointer to the data
* Input:  dataSize: The number of bytes of the data
* Return: true or false
*/
bool GenericList::isCached(void * ptrData,size_t sizeData)
	{
	if (this->cachedItem==NULL)	return false;
	return this->cachedItem->contains(ptrData,sizeData);
	}

/*
* Set the local cache content with the specified data
* Input:  index  : The index of the cached Item
* Input:  ptrItem: The Item
* Return: none
*/

void GenericList::setCache(long index,GenericListItem * ptrItem)
	{
	this->cachedIndex=index;
	this->cachedItem =ptrItem;
	}

/*
* Clear the local cache content
*/
void GenericList::clearCache(void)
	{
	this->cachedIndex=-1;
	this->cachedItem =NULL;
	}
