/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/


// Copyright (C) 2002  Farid Bengrid


#ifndef _GENERICLIST_HPP_
#define _GENERICLIST_HPP_

#ifndef _GENERICLISTITEM_HPP_
#include "GenericListItem.hpp"
#endif

#ifndef _ERRMSG_H_
#include "..\Shared\ErrMsg.h"
#endif

/*
* List
*/
class GenericList
	{
	public:
		/*
		* Contructor
		*/
		GenericList(void);

		/*
		* Destructor
		*/
		~GenericList(void);

		/*
		* Add an element at the end of the list
		* Input:  ptrData : A pointer to the data to store.
		* Input:  dataSize: The number of bytes to be copied.
		* Return: 0                : Successfull.
		*         ERR_MEMORYFAILURE: Allocation memory failure
		*         ERR_FUNCTIONPARAM: sizeData == 0 and ptrData!=NULL
		*/
		int add(void * ptrData,size_t sizeData);

		/*
		* Insert a element at the position specified of the list
		* Input: index: index of the insertion => 0 to this->Size()
		* Input: ptrData : A pointer to the data to add
		* Input: sizeData: number of bytes to be copied
		* Return: 0: Successfull.
		*         ERR_FUNCTIONPARAM: index <0 or (sizeData == 0 and ptrData!=NULL)
		*         ERR_MEMORYFAILURE: Allocation memory failure
		*/
		int insertAt(long index, void * ptrData,size_t sizeData);

		/*
		* Update an Item with the content specified
		* Input:  index  : the index of the item
		* Input:  ptrData : A pointer to the data
		* Input:  dataSize: The number of bytes of the data
		* Return: 0                : Successfull.
		*         ERR_FUNCTIONPARAM: Incorrect index specified or (datasize == 0 and ptrData!=NULL)
		*         ERR_MEMORYFAILURE: Memory allocation failure
		*/
		int update(long index, void * ptrData, size_t sizeData);
		
		/*
		* Get a copy of the Item Data at the specified position
		* Input:  index  : the index of the item
		* Output: ptrData: A pointer to a pre-allocated memory where the data will be copied
		* Return: 0                : Successfull.
		* Return: 0                        : Successfull.
		*         ERR_FUNCTIONPARAM        : Incorrect index specified, or ptrData==NULL
		*         ERR_LISTITEM_MISSING_DATA: There are no stored data.
		*/
		int get(long index, void * ptrData);

		/*
		* Get a Pointer to the data for Read Access (DO NOT CHANGE THE CONTENT !!!)
		* Input: index  : the index of the item
		* Return: void *: Pointer to the Data
		*           NULL: no Data,or index not found.
		*/
		void * getPtrToData(long index);

		/*
		* Remove the Item at the index specified
		* Input:  index: the index of the item
		* Return: 0                : Successfull.
		*         ERR_FUNCTIONPARAM: Incorrect index specified
		*/
		int remove(long index);

		/*
		* Remove the first element matching the one specified found in the list
		* Input:  ptrData : A pointer to the data
		* Input:  dataSize: The number of bytes of the data
		* Return: 0                      : Successfull.
		*         ERR_LIST_ITEM_NOT_FOUND: Item not found
		*/
		int remove(void * ptrData,size_t sizeData);

		/*
		Clear all the elements from the List
		Input : none
		Return: none
		*/
		void clear(void);
		/*
		* Get the size in bytes of the element at the position specified
		* Input: index  : Position of the element => 0 to this->Size()-1
		* return: size_t: the number of bytes.
		*              0: Item not found
		*/
		size_t getDataSize(long index);
		
		/*
		* Return the index in the list of the first element matching the one specified
		* Input:  ptrData : A pointer to the data
		* Input:  dataSize: The number of bytes of the data
		* Return: long                     : The index
		*         ERR_LIST_ITEM_NOT_FOUND  : Item not found
		*/
		long indexOf(void * ptrData,size_t sizeData);

		/*
		* Check if there is in the list an element matching to the one specified,
		* Input:  ptrData : A pointer to the data
		* Input:  dataSize: The number of bytes of the data
		* Return: true or false
		*/
		bool contains(void * ptrData,size_t sizeData);

		/*
		* Return the number of elements in the list
		* Input : none
		* return: long : the number of elements
		*/
		long size(void);

	private:
		GenericListItem * itemRoot;
		GenericListItem * itemLast;
		long nbItems;
		
		long              cachedIndex;
		GenericListItem * cachedItem;
		int LastError;
		
		/*
		* Create a new Item
		* Input:  ptrData : A pointer to the data to store.
		* Input:  dataSize: The number of bytes to be copied.
		* Return: GenericListItem * : A pointer to the Item
		*                       NULL: this->LastError=ERR_MEMORYFAILURE: Allocation memory failure
		*                       NULL: This->LastError=ERR_FUNCTIONPARAM: sizeData == 0 and ptrData!=NULL
		*/
		GenericListItem * createNewItem(void * ptrData,size_t sizeData);

		/*
		* Search an Item by its index
		* Input:  index: The index
		* Return: GenericListItem * : the Item
		*/
		GenericListItem * searchItem(long index);

		/*
		* Search an Item by its content
		* Input:  ptrData : A pointer to the data
		* Input:  dataSize: The number of bytes of the data
		* Output:    index: The index of the Item found
		* Return: GenericListItem * : the Item
		*                       NULL: Item not found
		*/
		GenericListItem * searchItem(long * index,void * ptrData,size_t sizeData);
		
		
		/*
		* Remove all the reference of an item from the list
		* Input:  Item: The Item
		* Return: none
		*/
		void extractFromList(GenericListItem * item);

		/*
		* Chech if the cached item correspond to the specified index
		* Input:  index: the index looked for
		* Return: true or false
		*/
		bool isCached(long index);

		/*
		* Chech if the cached item contain the specified data
		* Input:  ptrData : A pointer to the data
		* Input:  dataSize: The number of bytes of the data
		* Return: true or false
		*/
		bool isCached(void * ptrData,size_t sizeData);
		
		/*
		* Set the local cache content with the specified data
		* Input:  index  : The index of the cached Item
		* Input:  ptrItem: The Item
		* Return: none
		*/
		void setCache(long index,GenericListItem * ptrItem);
		
		/*
		* Clear the local cache content
		*/
		void clearCache(void);
	};

#endif