/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/


// Copyright (C) 2002  Farid Bengrid


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "IniHandler.hpp"

/*
* Constructor
*/
IniHandler::IniHandler()
	{
	this->iniFilename=NULL;
	this->listSections=new GenericList();
	}

/*
* Destructor
*/
IniHandler::~IniHandler()
	{
	if (this->iniFilename!=NULL)
		free(this->iniFilename);
	
	delete this->listSections;
	}

	
/*
* Add (or update) an item value (section & item will be created if not existing)
* Input: ptrSectionName: Pointer to the section name
* Input: ptrItemName   : Pointer to the item name
* Input: ptrItemValue  : Pointer to the value content
* Return: 0                : Successfull
*         ERR_MEMORYFAILURE: Allocation memory failure
*/
int IniHandler::add(char * ptrSectionName, char * ptrItemName, char * ptrItemValue)
	{
	bool newSection=false;
	IniSection * ptrSection;
	
	//Get a Pointer to the Section
	if((ptrSection=getSection(ptrSectionName))==NULL)
		{
		ptrSection=new IniSection();
		if(ptrSection->setName(ptrSectionName)!=0)
			{
			delete ptrSection;
			return ERR_MEMORYFAILURE;
			}
		
		newSection=true;
		}
	//Add the Item
	if(ptrSection->Items->addItem(ptrItemName,ptrItemValue)!=0)
		{
		if (newSection)
			delete ptrSection;
		return ERR_MEMORYFAILURE;
		}

	if(newSection)
		{
		if (this->listSections->add(&ptrSection,sizeof(IniSection *))!=0)
			{
			delete ptrSection;
			return ERR_MEMORYFAILURE;
			}
		}
	
	return 0;
	}

/*
* Get the number of character of the Item Value
* Input: ptrSectionName: Pointer to the section name
* Input: ptrItemName   : Pointer to the item name
* Return: int                     : Successfull
*         ERR_INI_SECTION_NOTFOUND: Section not found
*         ERR_INI_ITEM_NOTFOUND   : Item not Found
*/
int IniHandler::getItemValueLength(char * ptrSectionName, char * ptrItemName)
	{
	IniSection * ptrSection;
	
	//Get a Pointer to the Section
	if((ptrSection=getSection(ptrSectionName))==NULL)		return ERR_INI_SECTION_NOTFOUND;
	
	return ptrSection->Items->getItemValueLength(ptrItemName);
	}

/*
* Remove all the local entries saved in memory.
* Input: None.
* Return:None.
*/
void IniHandler::Clear(void)
	{
	if (this->iniFilename!=NULL)
		free(this->iniFilename);
	this->iniFilename=NULL;
	this->listSections->clear();
	}

/*
* Get a copy of the Item Value
* Input: ptrSectionName: Pointer to the section name
* Input: ptrItemName   : Pointer to the item name
* Output:ptrItemValue  : Pointer to where the value will be copied
* Return: 0                        : Successfull.
*         ERR_INI_SECTION_NOTFOUND : Section not found
*         ERR_INI_ITEM_NOTFOUND    : Item not Found
*         ERR_INI_FIELD_EMPTY      : the Item Value is empty
*/
int IniHandler::getItemValue(char * ptrSectionName, char * ptrItemName, char * ptrItemValue)
	{
	IniSection * ptrSection;
	
	//Get a Pointer to the Section
	if((ptrSection=getSection(ptrSectionName))==NULL)		return ERR_INI_SECTION_NOTFOUND;

	return ptrSection->Items->getItemValue(ptrItemName,ptrItemValue);
	}

/*
* Read Ini items from the specified file
* Input: filename: The ini file 
* Return: int               : The number of Item loaded
*         ERR_INI_FILEERROR : Can not find/open the file specified
*         ERR_MEMORYFAILURE : Allocation memory failure
*/
int IniHandler::readFromFile(char * filename)
	{
	FILE * hFile;
	char Section[MAX_BUFFER_READ];
	char Buffer[MAX_BUFFER_READ];
	int Context;
	int idx;
	int idxSeparator;
	int ErrCode;
	int NbEntries;


	if((hFile=fopen(filename,"rt"))==NULL)	return ERR_INI_FILEERROR;

	//Set the Filename to this instance
	if(setFilename(filename)!=0)
		{
		fclose(hFile);
		return ERR_MEMORYFAILURE;
		}

	NbEntries=0;

	//Clear local Data
	this->listSections->clear();
	
	while((Context=getNextFromFile(hFile,Buffer))!=CTX_NOMORE)
		{
		switch(Context)
			{
			case CTX_SECTION:	Buffer[strlen(Buffer)-1]=0x00;
									sprintf(Section,"%s",Buffer+1);
									break;
			case CTX_ITEM:		if (strlen(Section)>0)
										{
										for (idxSeparator=0    ;Buffer[idxSeparator]!='=';idxSeparator++);
										for (idx=idxSeparator-1;Buffer[idx]==' ';Buffer[idx]=0x00,idx--);
										for (idxSeparator++    ;Buffer[idxSeparator]==' ';idxSeparator++);
										if ((ErrCode=add(Section,Buffer,Buffer+idxSeparator))!=0)
											{
											fclose(hFile);
											this->listSections->clear();
											return ErrCode;
											}
										NbEntries++;
										}
									break;
			}
		}
	fclose(hFile);
	return NbEntries;
	}

/*
* Save the items contained in memory to a previously opened ini file
* Input: none
* Return: 0                 : Successfull
*         ERR_INI_NOFILENAME: No ini file was previously opened...
*         ERR_INI_FILEERROR : Can not find/open the file specified
*/
int IniHandler::save(void)
	{
	IniSection * ptrSection;
	FILE * hf;
	long idxSection;
	long idxItem;
	char * itemName;
	char * itemValue;

	if (this->iniFilename==NULL)	return ERR_INI_NOFILENAME;
	if((hf=fopen(this->iniFilename,"wt"))==NULL)	return ERR_INI_FILEERROR;

	for (idxSection=0;idxSection<this->listSections->size();idxSection++)
		{
		this->listSections->get(idxSection,&ptrSection);

		//Add the Section
		fprintf(hf,"[%s]\n",ptrSection->getPtrName());

		//Add the Items
		for (idxItem=0;idxItem<ptrSection->Items->getNumberOfItem();idxItem++)
			{
			itemName =NULL;
			itemValue=NULL;
			ptrSection->Items->getItemPtr(idxItem,&itemName,&itemValue);
			if (itemName!=NULL)
				fprintf(hf,"%s = %s\n",itemName,itemValue);

			}
		}
	fclose(hf);
	return 0;
	}

/*
* Save the items contained in memory to the specified ini file
* Input: filename: The ini file 
* Return: 0                 : Successfull
*         ERR_MEMORYFAILURE : Allocation memory failure
*         ERR_INI_NOFILENAME: No ini file was previously opened...
*         ERR_INI_FILEERROR : Can not find/open the file specified
*/
int IniHandler::save(char * filename)
	{
	FILE * hf;

	if (filename==NULL)	return ERR_INI_NOFILENAME;
	if((hf=fopen(filename,"wt"))==NULL)	return ERR_INI_FILEERROR;
	fclose(hf);

	//Set the Filename to this instance
	if(setFilename(filename)!=0)
		{
		fclose(hf);
		return ERR_MEMORYFAILURE;
		}

	return save();
	}

/*
* Return a pointer to the section defined by its name
* Input: ptrSectionName: A pointer to the name to search
* Return: IniSection * : The pointer to the Section
*                 NULL : not found
*/
IniSection * IniHandler::getSection(char * ptrSectionName)
	{
	IniSection * ptrSection=NULL;
	long index;
	
	for (index=0;index<this->listSections->size();index++)
		{
		this->listSections->get(index,&ptrSection);
		if (ptrSection->isEqualTo(ptrSectionName))
			return ptrSection;
		}
	return NULL;
	}

int IniHandler::getNextFromFile(FILE * hFile, char * Buffer)
	{
	unsigned int idx;
	memset(Buffer,0x00,MAX_BUFFER_READ*sizeof(char));
	while(fgets(Buffer,MAX_BUFFER_READ-1,hFile)!=NULL)
		{
		if (cleanBuffer(Buffer)>0)
			{
			if (Buffer[0]=='[')
				{
				//Section
				if (Buffer[strlen(Buffer)-1]==']')
					return CTX_SECTION;
				//Corrupted Section name....
				}
			else
				{
				//Search the = character
				for (idx=0;idx<strlen(Buffer) && Buffer[idx]!='=';idx++);
				if (idx<strlen(Buffer))
					return CTX_ITEM;
				//Corrupted Item...
				}
			}
		memset(Buffer,0x00,MAX_BUFFER_READ*sizeof(char));
		}
	return CTX_NOMORE;
	}

int IniHandler::cleanBuffer(char * Buffer)
	{
	unsigned int idx;
		
	for (idx=0;idx<strlen(Buffer) && Buffer[idx]==' ';idx++);
	if (idx>0)	strcpy(Buffer,Buffer+idx);
	for (idx=strlen(Buffer)-1;idx>0 && (Buffer[idx]==' ' || Buffer[idx]=='\r' || Buffer[idx]=='\n');idx--);
	if (idx>0)	Buffer[idx+1]=0x00;
	return strlen(Buffer);
	}

int IniHandler::setFilename(char * filename)
	{
	if (this->iniFilename!=NULL)
		free(this->iniFilename);
	
	this->iniFilename=(char *) calloc(strlen(filename)+1,sizeof(char));
	if (this->iniFilename==NULL)
		return -1;
	
	strcpy(this->iniFilename,filename);
	return 0;
	}